@@ -105,7 +105,7 @@ namespace xt
         {
             template <class E, class F>
             static std::ostream& output(std::ostream& out, const E& e, F& printer, std::size_t blanks,
-                                        std::size_t element_width, std::size_t edge_items, std::size_t line_width)
+                                        precision_type element_width, std::size_t edge_items, std::size_t line_width)
             {
                 using size_type = typename E::size_type;
 
@@ -168,7 +168,7 @@ namespace xt
         {
             template <class E, class F>
             static std::ostream& output(std::ostream& out, const E& e, F& printer,
-                                        std::size_t, std::size_t, std::size_t, std::size_t)
+                                        std::size_t, precision_type, std::size_t, std::size_t)
             {
                 if (e.dimension() == 0)
                 {
@@ -502,7 +502,7 @@ namespace xt
                 }
                 // insert j at end of number
                 std::size_t idx = s.find_last_not_of(" ");
-                s.insert(idx + 1, "j");
+                s.insert(idx + 1, "i");
                 out << s;
                 ++m_it;
                 return out;
@@ -632,6 +632,11 @@ namespace xt
         {
             lim = print_options::print_options().edge_items;
         }
+        if (sz == 0)
+        {
+            out << "{}";
+            return out;
+        }
 
         precision_type temp_precision = (precision_type)out.precision();
         precision_type precision = temp_precision;
