@@ -14,6 +14,8 @@
 #include <utility>
 
 #include "xexpression.hpp"
+#include "xiterable.hpp"
+#include "xlayout.hpp"
 
 namespace xt
 {
@@ -28,14 +30,34 @@ namespace xt
     class xscalar_stepper;
 
     template <bool is_const, class CT>
-    class xscalar_iterator;
+    class xdummy_iterator;
 
     template <class CT>
-    class xscalar : public xexpression<xscalar<CT>>
+    class xscalar;
+
+    template <class CT>
+    struct xiterable_inner_types<xscalar<CT>>
+    {
+        using value_type = std::decay_t<CT>;
+        using inner_shape_type = std::array<std::size_t, 0>;
+        using iterator = value_type*;
+        using const_iterator = const value_type*;
+        using const_stepper = xscalar_stepper<true, CT>;
+        using stepper = xscalar_stepper<false, CT>;
+        using reverse_iterator = std::reverse_iterator<iterator>;
+        using const_reverse_iterator = std::reverse_iterator<const_iterator>;
+
+    };
+
+    template <class CT>
+    class xscalar : public xexpression<xscalar<CT>>,
+                    public xiterable<xscalar<CT>>
     {
 
     public:
 
+        using self_type = xscalar<CT>;
+
         using value_type = std::decay_t<CT>;
         using reference = value_type&;
         using const_reference = const value_type&;
@@ -44,23 +66,28 @@ namespace xt
         using size_type = std::size_t;
         using difference_type = std::ptrdiff_t;
 
-        using self_type = xscalar<CT>;
-        using shape_type = std::array<size_type, 0>;
+        using iterable_base = xiterable<self_type>;
+        using inner_shape_type = typename iterable_base::inner_shape_type;
+        using shape_type = inner_shape_type;
 
-        using stepper = xscalar_stepper<false, CT>;
-        using const_stepper = xscalar_stepper<true, CT>;
+        using stepper = typename iterable_base::stepper;
+        using const_stepper = typename iterable_base::const_stepper;
 
-        using broadcast_iterator = xscalar_iterator<false, CT>;
-        using const_broadcast_iterator = xscalar_iterator<true, CT>;
+        using iterator = typename iterable_base::iterator;
+        using const_iterator = typename iterable_base::const_iterator;
 
-        using iterator = broadcast_iterator;
-        using const_iterator = const_broadcast_iterator;
+        using dummy_iterator = xdummy_iterator<false, CT>;
+        using const_dummy_iterator = xdummy_iterator<true, CT>;
+
+        static constexpr layout_type static_layout = layout_type::any;
+        static constexpr bool contiguous_layout = true;
 
         xscalar(CT value) noexcept;
 
         size_type size() const noexcept;
         size_type dimension() const noexcept;
         const shape_type& shape() const noexcept;
+        layout_type layout() const noexcept;
 
         template <class... Args>
         reference operator()(Args...) noexcept;
@@ -92,28 +119,6 @@ namespace xt
         const_iterator cbegin() const noexcept;
         const_iterator cend() const noexcept;
 
-        broadcast_iterator xbegin() noexcept;
-        broadcast_iterator xend() noexcept;
-
-        const_broadcast_iterator xbegin() const noexcept;
-        const_broadcast_iterator xend() const noexcept;
-        const_broadcast_iterator cxbegin() const noexcept;
-        const_broadcast_iterator cxend() const noexcept;
-
-        template <class S>
-        broadcast_iterator xbegin(const S& shape) noexcept;
-        template <class S>
-        broadcast_iterator xend(const S& shape) noexcept;
-
-        template <class S>
-        const_broadcast_iterator xbegin(const S& shape) const noexcept;
-        template <class S>
-        const_broadcast_iterator xend(const S& shape) const noexcept;
-        template <class S>
-        const_broadcast_iterator cxbegin(const S& shape) const noexcept;
-        template <class S>
-        const_broadcast_iterator cxend(const S& shape) const noexcept;
-
         template <class S>
         stepper stepper_begin(const S& shape) noexcept;
         template <class S>
@@ -124,6 +129,12 @@ namespace xt
         template <class S>
         const_stepper stepper_end(const S& shape) const noexcept;
 
+        dummy_iterator dummy_begin() noexcept;
+        dummy_iterator dummy_end() noexcept;
+
+        const_dummy_iterator dummy_begin() const noexcept;
+        const_dummy_iterator dummy_end() const noexcept;
+
     private:
 
         CT m_value;
@@ -167,7 +178,9 @@ namespace xt
         void step(size_type dim, size_type n = 1) noexcept;
         void step_back(size_type dim, size_type n = 1) noexcept;
         void reset(size_type dim) noexcept;
+        void reset_back(size_type dim) noexcept;
 
+        void to_begin() noexcept;
         void to_end() noexcept;
 
         bool equal(const self_type& rhs) const noexcept;
@@ -186,16 +199,16 @@ namespace xt
                     const xscalar_stepper<is_const, CT>& rhs) noexcept;
 
     /********************
-     * xscalar_iterator *
+     * xdummy_iterator *
      ********************/
 
     template <bool is_const, class CT>
-    class xscalar_iterator
+    class xdummy_iterator
     {
 
     public:
 
-        using self_type = xscalar_iterator<is_const, CT>;
+        using self_type = xdummy_iterator<is_const, CT>;
         using container_type = std::conditional_t<is_const,
                                                   const xscalar<CT>,
                                                   xscalar<CT>>;
@@ -210,7 +223,7 @@ namespace xt
         using difference_type = typename container_type::difference_type;
         using iterator_category = std::forward_iterator_tag;
 
-        explicit xscalar_iterator(container_type* c) noexcept;
+        explicit xdummy_iterator(container_type* c) noexcept;
 
         self_type& operator++() noexcept;
         self_type operator++(int) noexcept;
@@ -225,12 +238,43 @@ namespace xt
     };
 
     template <bool is_const, class CT>
-    bool operator==(const xscalar_iterator<is_const, CT>& lhs,
-                    const xscalar_iterator<is_const, CT>& rhs) noexcept;
+    bool operator==(const xdummy_iterator<is_const, CT>& lhs,
+                    const xdummy_iterator<is_const, CT>& rhs) noexcept;
 
     template <bool is_const, class CT>
-    bool operator!=(const xscalar_iterator<is_const, CT>& lhs,
-                    const xscalar_iterator<is_const, CT>& rhs) noexcept;
+    bool operator!=(const xdummy_iterator<is_const, CT>& lhs,
+                    const xdummy_iterator<is_const, CT>& rhs) noexcept;
+
+    /*******************************
+     * trivial_begin / trivial_end *
+     *******************************/
+
+    namespace detail
+    {
+        template <class CT>
+        constexpr auto trivial_begin(xscalar<CT>& c) -> decltype(c.dummy_begin())
+        {
+            return c.dummy_begin();
+        }
+
+        template <class CT>
+        constexpr auto trivial_end(xscalar<CT>& c) -> decltype(c.dummy_end())
+        {
+            return c.dummy_end();
+        }
+
+        template <class CT>
+        constexpr auto trivial_begin(const xscalar<CT>& c) -> decltype(c.dummy_begin())
+        {
+            return c.dummy_begin();
+        }
+
+        template <class CT>
+        constexpr auto trivial_end(const xscalar<CT>& c) -> decltype(c.dummy_end())
+        {
+            return c.dummy_end();
+        }
+    }
 
     /**************************
      * xscalar implementation *
@@ -261,6 +305,12 @@ namespace xt
         return zero_shape;
     }
 
+    template <class CT>
+    inline layout_type xscalar<CT>::layout() const noexcept
+    {
+        return static_layout;
+    }
+
     template <class CT>
     template <class... Args>
     inline auto xscalar<CT>::operator()(Args...) noexcept -> reference
@@ -330,115 +380,37 @@ namespace xt
     template <class CT>
     inline auto xscalar<CT>::begin() noexcept -> iterator
     {
-        return iterator(this);
+        return &m_value;
     }
 
     template <class CT>
     inline auto xscalar<CT>::end() noexcept -> iterator
     {
-        return iterator(this);
+        return &m_value + 1;
     }
 
     template <class CT>
     inline auto xscalar<CT>::begin() const noexcept -> const_iterator
     {
-        return const_iterator(this);
+        return &m_value;
     }
 
     template <class CT>
     inline auto xscalar<CT>::end() const noexcept -> const_iterator
     {
-        return const_iterator(this);
+        return &m_value + 1;
     }
 
     template <class CT>
     inline auto xscalar<CT>::cbegin() const noexcept -> const_iterator
     {
-        return const_iterator(this);
+        return &m_value;
     }
 
     template <class CT>
     inline auto xscalar<CT>::cend() const noexcept -> const_iterator
     {
-        return const_iterator(this);
-    }
-
-    template <class CT>
-    inline auto xscalar<CT>::xbegin() noexcept -> broadcast_iterator
-    {
-        return broadcast_iterator(this);
-    }
-
-    template <class CT>
-    inline auto xscalar<CT>::xend() noexcept -> broadcast_iterator
-    {
-        return broadcast_iterator(this);
-    }
-
-    template <class CT>
-    inline auto xscalar<CT>::xbegin() const noexcept -> const_broadcast_iterator
-    {
-        return const_broadcast_iterator(this);
-    }
-
-    template <class CT>
-    inline auto xscalar<CT>::xend() const noexcept -> const_broadcast_iterator
-    {
-        return const_broadcast_iterator(this);
-    }
-
-    template <class CT>
-    inline auto xscalar<CT>::cxbegin() const noexcept -> const_broadcast_iterator
-    {
-        return xbegin();
-    }
-
-    template <class CT>
-    inline auto xscalar<CT>::cxend() const noexcept -> const_broadcast_iterator
-    {
-        return xend();
-    }
-
-    template <class CT>
-    template <class S>
-    inline auto xscalar<CT>::xbegin(const S&) noexcept -> broadcast_iterator
-    {
-        return broadcast_iterator(this);
-    }
-
-    template <class CT>
-    template <class S>
-    inline auto xscalar<CT>::xend(const S&) noexcept -> broadcast_iterator
-    {
-        return broadcast_iterator(this);
-    }
-
-    template <class CT>
-    template <class S>
-    inline auto xscalar<CT>::xbegin(const S&) const noexcept -> const_broadcast_iterator
-    {
-        return const_broadcast_iterator(this);
-    }
-
-    template <class CT>
-    template <class S>
-    inline auto xscalar<CT>::xend(const S&) const noexcept -> const_broadcast_iterator
-    {
-        return const_broadcast_iterator(this);
-    }
-
-    template <class CT>
-    template <class S>
-    inline auto xscalar<CT>::cxbegin(const S&) const noexcept -> const_broadcast_iterator
-    {
-        return const_broadcast_iterator(this);
-    }
-
-    template <class CT>
-    template <class S>
-    inline auto xscalar<CT>::cxend(const S&) const noexcept -> const_broadcast_iterator
-    {
-        return const_broadcast_iterator(this);
+        return &m_value + 1;
     }
 
     template <class CT>
@@ -452,7 +424,7 @@ namespace xt
     template <class S>
     inline auto xscalar<CT>::stepper_end(const S&) noexcept -> stepper
     {
-        return stepper(this);
+        return stepper(this + 1);
     }
 
     template <class CT>
@@ -469,6 +441,30 @@ namespace xt
         return const_stepper(this + 1);
     }
 
+    template <class CT>
+    inline auto xscalar<CT>::dummy_begin() noexcept -> dummy_iterator
+    {
+        return dummy_iterator(this);
+    }
+
+    template <class CT>
+    inline auto xscalar<CT>::dummy_end() noexcept -> dummy_iterator
+    {
+        return dummy_iterator(this);
+    }
+
+    template <class CT>
+    inline auto xscalar<CT>::dummy_begin() const noexcept -> const_dummy_iterator
+    {
+        return const_dummy_iterator(this);
+    }
+
+    template <class CT>
+    inline auto xscalar<CT>::dummy_end() const noexcept -> const_dummy_iterator
+    {
+        return const_dummy_iterator(this);
+    }
+
     template <class T>
     inline xscalar<T&> xref(T& t)
     {
@@ -512,6 +508,17 @@ namespace xt
     {
     }
 
+    template <bool is_const, class CT>
+    inline void xscalar_stepper<is_const, CT>::reset_back(size_type /*dim*/) noexcept
+    {
+    }
+
+    template <bool is_const, class CT>
+    inline void xscalar_stepper<is_const, CT>::to_begin() noexcept
+    {
+        p_c = p_c->stepper_begin(p_c->shap()).pc;
+    }
+
     template <bool is_const, class CT>
     inline void xscalar_stepper<is_const, CT>::to_end() noexcept
     {
@@ -539,23 +546,23 @@ namespace xt
     }
 
     /***********************************
-     * xscalar_iterator implementation *
+     * xdummy_iterator implementation *
      ***********************************/
 
     template <bool is_const, class CT>
-    inline xscalar_iterator<is_const, CT>::xscalar_iterator(container_type* c) noexcept
+    inline xdummy_iterator<is_const, CT>::xdummy_iterator(container_type* c) noexcept
         : p_c(c)
     {
     }
 
     template <bool is_const, class CT>
-    inline auto xscalar_iterator<is_const, CT>::operator++() noexcept -> self_type&
+    inline auto xdummy_iterator<is_const, CT>::operator++() noexcept -> self_type&
     {
         return *this;
     }
 
     template <bool is_const, class CT>
-    inline auto xscalar_iterator<is_const, CT>::operator++(int) noexcept -> self_type
+    inline auto xdummy_iterator<is_const, CT>::operator++(int) noexcept -> self_type
     {
         self_type tmp(*this);
         ++(*this);
@@ -563,27 +570,27 @@ namespace xt
     }
 
     template <bool is_const, class CT>
-    inline auto xscalar_iterator<is_const, CT>::operator*() const noexcept -> reference
+    inline auto xdummy_iterator<is_const, CT>::operator*() const noexcept -> reference
     {
         return p_c->operator()();
     }
 
     template <bool is_const, class CT>
-    inline bool xscalar_iterator<is_const, CT>::equal(const self_type& rhs) const noexcept
+    inline bool xdummy_iterator<is_const, CT>::equal(const self_type& rhs) const noexcept
     {
         return p_c == rhs.p_c;
     }
 
     template <bool is_const, class CT>
-    inline bool operator==(const xscalar_iterator<is_const, CT>& lhs,
-                           const xscalar_iterator<is_const, CT>& rhs) noexcept
+    inline bool operator==(const xdummy_iterator<is_const, CT>& lhs,
+                           const xdummy_iterator<is_const, CT>& rhs) noexcept
     {
         return lhs.equal(rhs);
     }
 
     template <bool is_const, class CT>
-    inline bool operator!=(const xscalar_iterator<is_const, CT>& lhs,
-                           const xscalar_iterator<is_const, CT>& rhs) noexcept
+    inline bool operator!=(const xdummy_iterator<is_const, CT>& lhs,
+                           const xdummy_iterator<is_const, CT>& rhs) noexcept
     {
         return !(lhs.equal(rhs));
     }
