@@ -15,6 +15,7 @@
 #include <utility>
 #include <vector>
 
+#include "xbuffer_adaptor.hpp"
 #include "xcontainer.hpp"
 #include "xsemantic.hpp"
 
@@ -79,12 +80,15 @@ namespace xt
 
         xtensor_container();
         xtensor_container(nested_initializer_list_t<value_type, N> t);
-        explicit xtensor_container(const shape_type& shape, layout_type l = layout_type::row_major);
-        explicit xtensor_container(const shape_type& shape, const_reference value, layout_type l = layout_type::row_major);
+        explicit xtensor_container(const shape_type& shape, layout_type l = L);
+        explicit xtensor_container(const shape_type& shape, const_reference value, layout_type l = L);
         explicit xtensor_container(const shape_type& shape, const strides_type& strides);
         explicit xtensor_container(const shape_type& shape, const strides_type& strides, const_reference value);
         explicit xtensor_container(container_type&& data, inner_shape_type&& shape, inner_strides_type&& strides);
 
+        template <class S = shape_type>
+        static xtensor_container from_shape(S&& s);
+
         ~xtensor_container() = default;
 
         xtensor_container(const xtensor_container&) = default;
@@ -162,9 +166,11 @@ namespace xt
         using shape_type = typename base_type::shape_type;
         using strides_type = typename base_type::strides_type;
 
-        xtensor_adaptor(container_type& data);
-        xtensor_adaptor(container_type& data, const shape_type& shape, layout_type l = layout_type::row_major);
-        xtensor_adaptor(container_type& data, const shape_type& shape, const strides_type& strides);
+        using container_closure_type = adaptor_closure_t<container_type>;
+
+        xtensor_adaptor(container_closure_type data);
+        xtensor_adaptor(container_closure_type data, const shape_type& shape, layout_type l = layout_type::row_major);
+        xtensor_adaptor(container_closure_type data, const shape_type& shape, const strides_type& strides);
 
         ~xtensor_adaptor() = default;
 
@@ -179,7 +185,7 @@ namespace xt
 
     private:
 
-        container_type& m_data;
+        container_closure_type m_data;
 
         container_type& data_impl() noexcept;
         const container_type& data_impl() const noexcept;
@@ -286,6 +292,18 @@ namespace xt
         : base_type(std::move(shape), std::move(strides)), m_data(std::move(data))
     {
     }
+
+    template <class EC, std::size_t N, layout_type L>
+    template <class S>
+    inline xtensor_container<EC, N, L> xtensor_container<EC, N, L>::from_shape(S&& s)
+    {
+        if (s.size() != N)
+        {
+            throw std::runtime_error("Cannot change dimension of xtensor.");
+        }
+        shape_type shape = forward_sequence<shape_type>(s);
+        return self_type(shape);
+    }
     //@}
 
     /**
@@ -345,8 +363,8 @@ namespace xt
      * @param data the container to adapt
      */
     template <class EC, std::size_t N, layout_type L>
-    inline xtensor_adaptor<EC, N, L>::xtensor_adaptor(container_type& data)
-        : base_type(), m_data(data)
+    inline xtensor_adaptor<EC, N, L>::xtensor_adaptor(container_closure_type data)
+        : base_type(), m_data(std::forward<container_closure_type>(data))
     {
     }
 
@@ -358,8 +376,8 @@ namespace xt
      * @param l the layout_type of the xtensor_adaptor
      */
     template <class EC, std::size_t N, layout_type L>
-    inline xtensor_adaptor<EC, N, L>::xtensor_adaptor(container_type& data, const shape_type& shape, layout_type l)
-        : base_type(), m_data(data)
+    inline xtensor_adaptor<EC, N, L>::xtensor_adaptor(container_closure_type data, const shape_type& shape, layout_type l)
+        : base_type(), m_data(std::forward<container_closure_type>(data))
     {
         base_type::reshape(shape, l);
     }
@@ -372,8 +390,8 @@ namespace xt
      * @param strides the strides of the xtensor_adaptor
      */
     template <class EC, std::size_t N, layout_type L>
-    inline xtensor_adaptor<EC, N, L>::xtensor_adaptor(container_type& data, const shape_type& shape, const strides_type& strides)
-        : base_type(), m_data(data)
+    inline xtensor_adaptor<EC, N, L>::xtensor_adaptor(container_closure_type data, const shape_type& shape, const strides_type& strides)
+        : base_type(), m_data(std::forward<container_closure_type>(data))
     {
         base_type::reshape(shape, strides);
     }
