@@ -262,7 +262,7 @@ namespace xt
         R apply(std::size_t index, F&& func, std::index_sequence<I...> /*seq*/, const std::tuple<S...>& s) noexcept(noexcept(std::declval<F>()))
         {
             using FT = std::add_pointer_t<R(F&&, const std::tuple<S...>&)>;
-            static const std::array<FT, sizeof...(I)> ar = {&apply_one<R, F, I, S...>...};
+            static const std::array<FT, sizeof...(I)> ar = {{&apply_one<R, F, I, S...>...}};
             return ar[index](std::forward<F>(func), s);
         }
     }
@@ -897,6 +897,23 @@ namespace xt
     {
         return N;
     }
+
+    /*****************************************
+     * has_raw_data_interface implementation *
+     *****************************************/
+
+    template <typename T>
+    class has_raw_data_interface
+    {
+        template <typename C>
+        static std::true_type test(decltype(std::declval<C>().raw_data_offset()));
+
+        template <typename C>
+        static std::false_type test(...);
+
+    public:
+        constexpr static bool value = decltype(test<T>(std::size_t(0)))::value == true;
+    };
 }
 
 #endif
